!SURFEX_LIC Copyright 1994-2014 Meteo-France 
!SURFEX_LIC This is part of the SURFEX software governed by the CeCILL-C  licence
!SURFEX_LIC version 1. See LICENSE, CeCILL-C_V1-en.txt and CeCILL-C_V1-fr.txt
!SURFEX_LIC for details. version 1.
!     #####################
      MODULE MODD_CH_WATFLUX_n
!     ######################
!
!!
!!    PURPOSE
!!    -------
!     
!   
!
!!
!!**  IMPLICIT ARGUMENTS
!!    ------------------
!!      None
!!
!
!!    AUTHOR
!!    ------
!!  P. Tulet   *Meteo France*
!!
!!    MODIFICATIONS
!!    -------------
!!  16/07/03 (P. Tulet)  restructured for externalization
!------------------------------------------------------------------------------
!
!*       0.   DECLARATIONS
!             ------------
!
!
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK
USE PARKIND1  ,ONLY : JPRB
!
IMPLICIT NONE

TYPE CH_WATFLUX_t
!
  CHARACTER(LEN=6)              :: CCH_DRY_DEP              ! deposition scheme
  REAL, DIMENSION(:,:), POINTER :: XDEP =>NULL()                    ! final dry deposition
                                                            ! velocity  for lakes
  CHARACTER(LEN=6), DIMENSION(:), POINTER :: CSV =>NULL()           ! name of the scalar var.
  INTEGER    :: NSV_CHSBEG, NSV_CHSEND                      ! chemical begin and ending
                                                            ! index of the HSV/CSV array
  INTEGER    :: NBEQ                                        ! number of chemical species
                                                            ! in the surface scheme

  INTEGER    :: NSV_DSTBEG, NSV_DSTEND                      ! dust begin and ending
  INTEGER    :: NSV_SLTBEG, NSV_SLTEND                      ! sea salt begin and ending
  INTEGER    :: NSV_AERBEG, NSV_AEREND                      ! aerosol begin and ending
  INTEGER    :: NDSTEQ                                      ! number of dust species
  INTEGER    :: NSLTEQ                                      ! number of sea salt species
  INTEGER    :: NAEREQ                                      ! number of aerosol species

  CHARACTER(LEN=6), DIMENSION(:), POINTER :: CCH_NAMES=>NULL()      ! NAME OF CHEMICAL SPECIES
                                                            ! (FOR DIAG ONLY)
  CHARACTER(LEN=6), DIMENSION(:), POINTER :: CDSTNAMES=>NULL()
  CHARACTER(LEN=6), DIMENSION(:), POINTER :: CSLTNAMES=>NULL()
  CHARACTER(LEN=6), DIMENSION(:), POINTER :: CAER_NAMES=>NULL()


!
END TYPE CH_WATFLUX_t

TYPE(CH_WATFLUX_t), ALLOCATABLE, TARGET, SAVE :: CH_WATFLUX_MODEL(:)

 CHARACTER(LEN=6), POINTER :: CCH_DRY_DEP=>NULL()
!$OMP THREADPRIVATE(CCH_DRY_DEP)
REAL, DIMENSION(:,:), POINTER :: XDEP=>NULL()
!$OMP THREADPRIVATE(XDEP)
 CHARACTER(LEN=6), DIMENSION(:), POINTER :: CSV=>NULL()
!$OMP THREADPRIVATE(CSV)
INTEGER, POINTER :: NSV_CHSBEG=>NULL(), NSV_CHSEND=>NULL()
!$OMP THREADPRIVATE(NSV_CHSBEG, NSV_CHSEND)
INTEGER, POINTER :: NSV_DSTBEG=>NULL(), NSV_DSTEND=>NULL()
!$OMP THREADPRIVATE(NSV_DSTBEG, NSV_DSTEND)
INTEGER, POINTER :: NSV_SLTBEG=>NULL(), NSV_SLTEND=>NULL()
!$OMP THREADPRIVATE(NSV_SLTBEG, NSV_SLTEND)
INTEGER, POINTER :: NSV_AERBEG=>NULL(), NSV_AEREND=>NULL()
!$OMP THREADPRIVATE(NSV_AERBEG, NSV_AEREND)
INTEGER, POINTER :: NBEQ=>NULL()
!$OMP THREADPRIVATE(NBEQ)
INTEGER, POINTER :: NDSTEQ=>NULL()
!$OMP THREADPRIVATE(NDSTEQ)
INTEGER, POINTER :: NSLTEQ=>NULL()
!$OMP THREADPRIVATE(NSLTEQ)
INTEGER, POINTER :: NAEREQ=>NULL()
!$OMP THREADPRIVATE(NAEREQ)


 CHARACTER(LEN=6), DIMENSION(:), POINTER :: CCH_NAMES=>NULL()
!$OMP THREADPRIVATE(CCH_NAMES)
 CHARACTER(LEN=6), DIMENSION(:), POINTER :: CDSTNAMES=>NULL()
!$OMP THREADPRIVATE(CDSTNAMES)
 CHARACTER(LEN=6), DIMENSION(:), POINTER :: CSLTNAMES=>NULL()
!$OMP THREADPRIVATE(CSLTNAMES)
 CHARACTER(LEN=6), DIMENSION(:), POINTER :: CAER_NAMES=>NULL()
!$OMP THREADPRIVATE(CAER_NAMES)



CONTAINS

SUBROUTINE CH_WATFLUX_GOTO_MODEL(KFROM, KTO, LKFROM)
LOGICAL, INTENT(IN) :: LKFROM
INTEGER, INTENT(IN) :: KFROM, KTO
REAL(KIND=JPRB) :: ZHOOK_HANDLE
!
! Save current state for allocated arrays
IF (LKFROM) THEN
CH_WATFLUX_MODEL(KFROM)%XDEP=>XDEP
CH_WATFLUX_MODEL(KFROM)%CSV=>CSV
CH_WATFLUX_MODEL(KFROM)%CCH_NAMES=>CCH_NAMES
CH_WATFLUX_MODEL(KFROM)%CDSTNAMES=>CDSTNAMES
CH_WATFLUX_MODEL(KFROM)%CSLTNAMES=>CSLTNAMES
CH_WATFLUX_MODEL(KFROM)%CAER_NAMES=>CAER_NAMES
ENDIF

!
! Current model is set to model KTO
IF (LHOOK) CALL DR_HOOK('MODD_CH_WATFLUX_N:CH_WATFLUX_GOTO_MODEL',0,ZHOOK_HANDLE)
CCH_DRY_DEP=>CH_WATFLUX_MODEL(KTO)%CCH_DRY_DEP
XDEP=>CH_WATFLUX_MODEL(KTO)%XDEP
CSV=>CH_WATFLUX_MODEL(KTO)%CSV
NSV_CHSBEG=>CH_WATFLUX_MODEL(KTO)%NSV_CHSBEG
NSV_CHSEND=>CH_WATFLUX_MODEL(KTO)%NSV_CHSEND
NBEQ=>CH_WATFLUX_MODEL(KTO)%NBEQ
CCH_NAMES=>CH_WATFLUX_MODEL(KTO)%CCH_NAMES
NSV_DSTBEG=>CH_WATFLUX_MODEL(KTO)%NSV_DSTBEG
NSV_DSTEND=>CH_WATFLUX_MODEL(KTO)%NSV_DSTEND
NDSTEQ=>CH_WATFLUX_MODEL(KTO)%NDSTEQ
CDSTNAMES=>CH_WATFLUX_MODEL(KTO)%CDSTNAMES
NSV_SLTBEG=>CH_WATFLUX_MODEL(KTO)%NSV_SLTBEG
NSV_SLTEND=>CH_WATFLUX_MODEL(KTO)%NSV_SLTEND
NSLTEQ=>CH_WATFLUX_MODEL(KTO)%NSLTEQ
CSLTNAMES=>CH_WATFLUX_MODEL(KTO)%CSLTNAMES
NSV_AERBEG=>CH_WATFLUX_MODEL(KTO)%NSV_AERBEG
NSV_AEREND=>CH_WATFLUX_MODEL(KTO)%NSV_AEREND
NAEREQ=>CH_WATFLUX_MODEL(KTO)%NAEREQ
CAER_NAMES=>CH_WATFLUX_MODEL(KTO)%CAER_NAMES
IF (LHOOK) CALL DR_HOOK('MODD_CH_WATFLUX_N:CH_WATFLUX_GOTO_MODEL',1,ZHOOK_HANDLE)

END SUBROUTINE CH_WATFLUX_GOTO_MODEL

SUBROUTINE CH_WATFLUX_ALLOC(KMODEL)
INTEGER, INTENT(IN) :: KMODEL
REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK("MODD_CH_WATFLUX_N:CH_WATFLUX_ALLOC",0,ZHOOK_HANDLE)
ALLOCATE(CH_WATFLUX_MODEL(KMODEL))
CH_WATFLUX_MODEL(:)%CCH_DRY_DEP=' '
CH_WATFLUX_MODEL(:)%NSV_CHSBEG=0
CH_WATFLUX_MODEL(:)%NSV_CHSEND=0
CH_WATFLUX_MODEL(:)%NSV_DSTBEG=0
CH_WATFLUX_MODEL(:)%NSV_DSTEND=0
CH_WATFLUX_MODEL(:)%NBEQ=0
CH_WATFLUX_MODEL(:)%NSV_SLTBEG=0
CH_WATFLUX_MODEL(:)%NSV_SLTEND=0
CH_WATFLUX_MODEL(:)%NSV_AERBEG=0
CH_WATFLUX_MODEL(:)%NSV_AEREND=0
CH_WATFLUX_MODEL(:)%NDSTEQ=0
CH_WATFLUX_MODEL(:)%NSLTEQ=0
CH_WATFLUX_MODEL(:)%NAEREQ=0
IF (LHOOK) CALL DR_HOOK("MODD_CH_WATFLUX_N:CH_WATFLUX_ALLOC",1,ZHOOK_HANDLE)
END SUBROUTINE CH_WATFLUX_ALLOC

SUBROUTINE CH_WATFLUX_DEALLO
REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK("MODD_CH_WATFLUX_N:CH_WATFLUX_DEALLO",0,ZHOOK_HANDLE)
IF (ALLOCATED(CH_WATFLUX_MODEL)) DEALLOCATE(CH_WATFLUX_MODEL)
IF (LHOOK) CALL DR_HOOK("MODD_CH_WATFLUX_N:CH_WATFLUX_DEALLO",1,ZHOOK_HANDLE)
END SUBROUTINE CH_WATFLUX_DEALLO

END MODULE MODD_CH_WATFLUX_n
