!SURFEX_LIC Copyright 1994-2014 Meteo-France 
!SURFEX_LIC This is part of the SURFEX software governed by the CeCILL-C  licence
!SURFEX_LIC version 1. See LICENSE, CeCILL-C_V1-en.txt and CeCILL-C_V1-fr.txt
!SURFEX_LIC for details. version 1.
!     ############################
      MODULE MODD_DIAG_MISC_ISBA_n
!     ############################
!
!!****  *MODD_DIAG_MISC_ISBA - declaration of packed surface parameters for ISBA scheme
!!
!!    PURPOSE
!!    -------
!
!!
!!**  IMPLICIT ARGUMENTS
!!    ------------------
!!      None 
!!
!!    REFERENCE
!!    ---------
!!
!!    AUTHOR
!!    ------
!!	P. Le Moigne   *Meteo France*
!!
!!    MODIFICATIONS
!!    -------------
!!      Original       07/10/04
!!      A.L. Gibelin 04/2009 : Add respiration diagnostics
!!      A.L. Gibelin 05/2009 : Add carbon spinup
!!      A.L. Gibelin 07/2009 : Suppress RDK and transform GPP as a diagnostic
!!       B. Decharme 05/2012 : Carbon fluxes in diag_evap
!!       B. Decharme 07/2012 : New diag for DIF under LSURF_MISC_DIF key
!!                               F2 stress
!!                               Root zone swi, wg and wgi
!!                               swi, wg and wgi comparable to ISBA-FR-DG2 and DG3 layers
!!       B. Decharme 10/2012 : New diag for DIF 
!!                               active layer thickness over permafrost area
!!                               frozen layer thickness over non-permafrost area
!!
!-------------------------------------------------------------------------------
!
!*       0.   DECLARATIONS
!             ------------
!
!
!
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK
USE PARKIND1  ,ONLY : JPRB
!
IMPLICIT NONE

TYPE DIAG_MISC_ISBA_t
!------------------------------------------------------------------------------
!
  LOGICAL :: LSURF_MISC_BUDGET   ! flag for miscellaneous terms of isba scheme
  LOGICAL :: LSURF_DIAG_ALBEDO   ! flag to write out diagnostic albedo
  LOGICAL :: LSURF_MISC_DIF      ! flag for miscellaneous terms of isba-dif scheme
!
!* variables for each patch
!
  REAL, POINTER, DIMENSION(:,:) :: XHV           ! Halstead coefficient
!      
  REAL, POINTER, DIMENSION(:,:,:) :: XSWI        ! Soil wetness index
  REAL, POINTER, DIMENSION(:,:,:) :: XTSWI       ! Total soil wetness index
!     
  REAL, POINTER, DIMENSION(:,:)   :: XALT        ! Active layer thickness in permafrost area
  REAL, POINTER, DIMENSION(:,:)   :: XFLT        ! Frozen layer thickness in non-permmafrost area
!
  REAL, POINTER, DIMENSION(:,:,:) :: XSNOWLIQ    ! snow liquid water profile (ISBA-ES:3-L)
  REAL, POINTER, DIMENSION(:,:,:) :: XSNOWTEMP   ! snow temperature profile  (ISBA-ES:3-L)
!     
  REAL, POINTER, DIMENSION(:,:) :: XTWSNOW       ! Total snow reservoir
  REAL, POINTER, DIMENSION(:,:) :: XTDSNOW       ! Total snow height
  REAL, POINTER, DIMENSION(:,:) :: XTTSNOW       ! Total snow temperature
!
  REAL, POINTER, DIMENSION(:,:) :: XDPSNG         ! Snow fraction over ground, diag at time t
  REAL, POINTER, DIMENSION(:,:) :: XDPSNV         ! Snow fraction over vegetation, diag at time t
  REAL, POINTER, DIMENSION(:,:) :: XDPSN          ! Total Snow fraction, diag at time t
  REAL, POINTER, DIMENSION(:,:) :: XALBT          ! Total Albedo
!    
  REAL, POINTER, DIMENSION(:,:) :: XDFSAT         ! Topmodel/dt92 saturated fraction
!
  REAL, POINTER, DIMENSION(:,:) :: XDFFG          ! Flood fraction over ground, diag at time t
  REAL, POINTER, DIMENSION(:,:) :: XDFFV          ! Flood fraction over vegetation, diag at time t
  REAL, POINTER, DIMENSION(:,:) :: XDFF           ! Total Flood fraction, diag at time t
!
  REAL, POINTER, DIMENSION(:,:) :: XSEUIL        ! Irrigation threshold
!
  REAL, POINTER, DIMENSION(:,:) :: XFAPAR        ! Fapar of vegetation
  REAL, POINTER, DIMENSION(:,:) :: XFAPIR        ! Fapir of vegetation
  REAL, POINTER, DIMENSION(:,:) :: XDFAPARC      ! Fapar of vegetation (cumul)
  REAL, POINTER, DIMENSION(:,:) :: XDFAPIRC      ! Fapir of vegetation (cumul)
  REAL, POINTER, DIMENSION(:,:) :: XFAPAR_BS     ! Fapar of bare soil
  REAL, POINTER, DIMENSION(:,:) :: XFAPIR_BS     ! Fapir of bare soil
  REAL, POINTER, DIMENSION(:,:) :: XDLAI_EFFC    ! Effective LAI (cumul)
!  
!* average variables
!
  REAL, POINTER, DIMENSION(:)   :: XAVG_HV       ! Halstead coefficient
  REAL, POINTER, DIMENSION(:)   :: XAVG_LAI      ! leaf average index
!     
  REAL, POINTER, DIMENSION(:,:)   :: XAVG_SWI      ! Liquid Soil wetness index
  REAL, POINTER, DIMENSION(:,:)   :: XAVG_TSWI     ! Total soil wetness index
  REAL, POINTER, DIMENSION(:)     :: XSOIL_TSWI    ! Soil wetness index
  REAL, POINTER, DIMENSION(:)     :: XSOIL_TWG     ! Soil water content (liquid+ice)
  REAL, POINTER, DIMENSION(:)     :: XSOIL_TWGI    ! Soil ice content
!     
  REAL, POINTER, DIMENSION(:)   :: XAVG_ALT      ! Active layer thickness in permafrost area
  REAL, POINTER, DIMENSION(:)   :: XAVG_FLT      ! Frozen layer thickness in non-permmafrost area
!
  REAL, POINTER, DIMENSION(:)   :: XAVG_TWSNOW   ! Snow total reservoir
  REAL, POINTER, DIMENSION(:)   :: XAVG_TDSNOW   ! Snow total height
  REAL, POINTER, DIMENSION(:)   :: XAVG_TTSNOW   ! Snow total temperature
!
  REAL, POINTER, DIMENSION(:)   :: XAVG_PSNG     ! Snow fraction over ground
  REAL, POINTER, DIMENSION(:)   :: XAVG_PSNV     ! Snow fraction over vegetation
  REAL, POINTER, DIMENSION(:)   :: XAVG_PSN      ! Total Snow fraction
  REAL, POINTER, DIMENSION(:)   :: XAVG_ALBT     ! Total Albedo
!    
  REAL, POINTER, DIMENSION(:) :: XAVG_FSAT       ! Topmodel/dt92 saturated fraction
!      
  REAL, POINTER, DIMENSION(:) :: XAVG_FFG        ! Flood fraction over ground
  REAL, POINTER, DIMENSION(:) :: XAVG_FFV        ! Flood fraction over vegetation
  REAL, POINTER, DIMENSION(:) :: XAVG_FF         ! Total Flood fraction  
!
  REAL, POINTER, DIMENSION(:) :: XSURF_TSWI      ! Surface soil wetness index (DIF option)
  REAL, POINTER, DIMENSION(:) :: XSURF_TWG       ! Surface soil water content (liquid+ice) (DIF option)
  REAL, POINTER, DIMENSION(:) :: XSURF_TWGI      ! Surface soil ice content (DIF option)
  REAL, POINTER, DIMENSION(:) :: XROOT_TSWI      ! Root soil wetness index (DIF option)
  REAL, POINTER, DIMENSION(:) :: XROOT_TWG       ! Root soil water content (liquid+ice) (DIF option)
  REAL, POINTER, DIMENSION(:) :: XROOT_TWGI      ! Root soil ice content (DIF option)
  REAL, POINTER, DIMENSION(:) :: XFRD2_TSWI      ! ISBA-FR-DG2 comparable soil wetness index (DIF option)
  REAL, POINTER, DIMENSION(:) :: XFRD2_TWG       ! ISBA-FR-DG2 comparable soil water content (liquid+ice) (DIF option)
  REAL, POINTER, DIMENSION(:) :: XFRD2_TWGI      ! ISBA-FR-DG2 comparable soil ice content (DIF option)  
  REAL, POINTER, DIMENSION(:) :: XFRD3_TSWI      ! ISBA-FR-Deep comparable soil wetness index (DIF option)
  REAL, POINTER, DIMENSION(:) :: XFRD3_TWG       ! ISBA-FR-Deep comparable soil water content (liquid+ice) (DIF option)
  REAL, POINTER, DIMENSION(:) :: XFRD3_TWGI      ! ISBA-FR-Deep comparable soil ice content (DIF option)
!
!------------------------------------------------------------------------------
!

END TYPE DIAG_MISC_ISBA_t

TYPE(DIAG_MISC_ISBA_t), ALLOCATABLE, TARGET, SAVE :: DIAG_MISC_ISBA_MODEL(:)

LOGICAL, POINTER :: LSURF_MISC_BUDGET=>NULL()
!$OMP THREADPRIVATE(LSURF_MISC_BUDGET)
LOGICAL, POINTER :: LSURF_DIAG_ALBEDO=>NULL()
!$OMP THREADPRIVATE(LSURF_DIAG_ALBEDO)
LOGICAL, POINTER :: LSURF_MISC_DIF=>NULL()
!$OMP THREADPRIVATE(LSURF_MISC_DIF)

!      
REAL, POINTER, DIMENSION(:,:) :: XHV=>NULL()
!$OMP THREADPRIVATE(XHV)
REAL, POINTER, DIMENSION(:,:) :: XDPSNG=>NULL()
!$OMP THREADPRIVATE(XDPSNG)
REAL, POINTER, DIMENSION(:,:) :: XDPSNV=>NULL()
!$OMP THREADPRIVATE(XDPSNV)
REAL, POINTER, DIMENSION(:,:) :: XDPSN=>NULL()
!$OMP THREADPRIVATE(XDPSN)
REAL, POINTER, DIMENSION(:,:) :: XSEUIL=>NULL()
!$OMP THREADPRIVATE(XSEUIL)
REAL, POINTER, DIMENSION(:,:) :: XFAPAR=>NULL()
!$OMP THREADPRIVATE(XFAPAR)
REAL, POINTER, DIMENSION(:,:) :: XFAPIR=>NULL()
!$OMP THREADPRIVATE(XFAPIR)
REAL, POINTER, DIMENSION(:,:) :: XDFAPARC=>NULL()
!$OMP THREADPRIVATE(XDFAPARC)
REAL, POINTER, DIMENSION(:,:) :: XDFAPIRC=>NULL()
!$OMP THREADPRIVATE(XDFAPIRC)
REAL, POINTER, DIMENSION(:,:) :: XFAPAR_BS=>NULL()
!$OMP THREADPRIVATE(XFAPAR_BS)
REAL, POINTER, DIMENSION(:,:) :: XFAPIR_BS=>NULL()
!$OMP THREADPRIVATE(XFAPIR_BS)
REAL, POINTER, DIMENSION(:,:) :: XDLAI_EFFC=>NULL()
!$OMP THREADPRIVATE(XDLAI_EFFC)
!
REAL, POINTER, DIMENSION(:,:) :: XALBT=>NULL()
!$OMP THREADPRIVATE(XALBT)
REAL, POINTER, DIMENSION(:,:) :: XDFSAT=>NULL()
!$OMP THREADPRIVATE(XDFSAT)
REAL, POINTER, DIMENSION(:,:) :: XDFFG=>NULL()
!$OMP THREADPRIVATE(XDFFG)
REAL, POINTER, DIMENSION(:,:) :: XDFFV=>NULL()
!$OMP THREADPRIVATE(XDFFV)
REAL, POINTER, DIMENSION(:,:) :: XDFF=>NULL()
!$OMP THREADPRIVATE(XDFF)
!
REAL, POINTER, DIMENSION(:,:,:) :: XSWI=>NULL()
!$OMP THREADPRIVATE(XSWI)
REAL, POINTER, DIMENSION(:,:,:) :: XTSWI=>NULL()
!$OMP THREADPRIVATE(XTSWI)
REAL, POINTER, DIMENSION(:,:)   :: XALT=>NULL()
!$OMP THREADPRIVATE(XALT)
REAL, POINTER, DIMENSION(:,:)   :: XFLT=>NULL()
!$OMP THREADPRIVATE(XFLT)
!
REAL, POINTER, DIMENSION(:,:) :: XTWSNOW=>NULL()
!$OMP THREADPRIVATE(XTWSNOW)
REAL, POINTER, DIMENSION(:,:) :: XTDSNOW=>NULL()
!$OMP THREADPRIVATE(XTDSNOW)
REAL, POINTER, DIMENSION(:,:) :: XTTSNOW=>NULL()
!$OMP THREADPRIVATE(XTTSNOW)
!
REAL, POINTER, DIMENSION(:,:,:) :: XSNOWLIQ=>NULL()
!$OMP THREADPRIVATE(XSNOWLIQ)
REAL, POINTER, DIMENSION(:,:,:) :: XSNOWTEMP=>NULL()
!$OMP THREADPRIVATE(XSNOWTEMP)
!
REAL, POINTER, DIMENSION(:)   :: XAVG_HV=>NULL()
!$OMP THREADPRIVATE(XAVG_HV)
REAL, POINTER, DIMENSION(:)   :: XAVG_LAI=>NULL()
!$OMP THREADPRIVATE(XAVG_LAI)
REAL, POINTER, DIMENSION(:)   :: XAVG_PSNG=>NULL()
!$OMP THREADPRIVATE(XAVG_PSNG)
REAL, POINTER, DIMENSION(:)   :: XAVG_PSNV=>NULL()
!$OMP THREADPRIVATE(XAVG_PSNV)
REAL, POINTER, DIMENSION(:)   :: XAVG_PSN=>NULL()
!$OMP THREADPRIVATE(XAVG_PSN)
REAL, POINTER, DIMENSION(:)   :: XAVG_ALBT=>NULL()
!$OMP THREADPRIVATE(XAVG_ALBT)
REAL, POINTER, DIMENSION(:)   :: XAVG_FSAT=>NULL()
!$OMP THREADPRIVATE(XAVG_FSAT)
REAL, POINTER, DIMENSION(:)   :: XAVG_FFG=>NULL()
!$OMP THREADPRIVATE(XAVG_FFG)
REAL, POINTER, DIMENSION(:)   :: XAVG_FFV=>NULL()
!$OMP THREADPRIVATE(XAVG_FFV)
REAL, POINTER, DIMENSION(:)   :: XAVG_FF=>NULL()
!$OMP THREADPRIVATE(XAVG_FF)
!
REAL, POINTER, DIMENSION(:,:)   :: XAVG_SWI=>NULL()
!$OMP THREADPRIVATE(XAVG_SWI)
REAL, POINTER, DIMENSION(:,:)   :: XAVG_TSWI=>NULL()
!$OMP THREADPRIVATE(XAVG_TSWI)
REAL, POINTER, DIMENSION(:)     :: XSOIL_TSWI=>NULL()
!$OMP THREADPRIVATE(XSOIL_TSWI)
REAL, POINTER, DIMENSION(:)     :: XSOIL_TWG=>NULL()
!$OMP THREADPRIVATE(XSOIL_TWG)
REAL, POINTER, DIMENSION(:)     :: XSOIL_TWGI=>NULL()
!$OMP THREADPRIVATE(XSOIL_TWGI)
REAL, POINTER, DIMENSION(:)     :: XAVG_ALT=>NULL()
!$OMP THREADPRIVATE(XAVG_ALT)
REAL, POINTER, DIMENSION(:)     :: XAVG_FLT=>NULL()
!$OMP THREADPRIVATE(XAVG_FLT)
!
REAL, POINTER, DIMENSION(:) :: XAVG_TWSNOW=>NULL()
!$OMP THREADPRIVATE(XAVG_TWSNOW)
REAL, POINTER, DIMENSION(:) :: XAVG_TDSNOW=>NULL()
!$OMP THREADPRIVATE(XAVG_TDSNOW)
REAL, POINTER, DIMENSION(:) :: XAVG_TTSNOW=>NULL()
!$OMP THREADPRIVATE(XAVG_TTSNOW)
!
REAL, POINTER, DIMENSION(:) :: XSURF_TSWI=>NULL()
!$OMP THREADPRIVATE(XSURF_TSWI)
REAL, POINTER, DIMENSION(:) :: XSURF_TWG =>NULL()
!$OMP THREADPRIVATE(XSURF_TWG)
REAL, POINTER, DIMENSION(:) :: XSURF_TWGI=>NULL()
!$OMP THREADPRIVATE(XSURF_TWGI)
REAL, POINTER, DIMENSION(:) :: XROOT_TSWI=>NULL()
!$OMP THREADPRIVATE(XROOT_TSWI)
REAL, POINTER, DIMENSION(:) :: XROOT_TWG =>NULL()
!$OMP THREADPRIVATE(XROOT_TWG)
REAL, POINTER, DIMENSION(:) :: XROOT_TWGI=>NULL()
!$OMP THREADPRIVATE(XROOT_TWGI)
REAL, POINTER, DIMENSION(:) :: XFRD2_TSWI=>NULL()
!$OMP THREADPRIVATE(XFRD2_TSWI)
REAL, POINTER, DIMENSION(:) :: XFRD2_TWG =>NULL()
!$OMP THREADPRIVATE(XFRD2_TWG)
REAL, POINTER, DIMENSION(:) :: XFRD2_TWGI=>NULL()
!$OMP THREADPRIVATE(XFRD2_TWGI)
REAL, POINTER, DIMENSION(:) :: XFRD3_TSWI=>NULL()
!$OMP THREADPRIVATE(XFRD3_TSWI)
REAL, POINTER, DIMENSION(:) :: XFRD3_TWG =>NULL()
!$OMP THREADPRIVATE(XFRD3_TWG)
REAL, POINTER, DIMENSION(:) :: XFRD3_TWGI=>NULL()
!$OMP THREADPRIVATE(XFRD3_TWGI)
!
CONTAINS

SUBROUTINE DIAG_MISC_ISBA_GOTO_MODEL(KFROM, KTO, LKFROM)
LOGICAL, INTENT(IN) :: LKFROM
INTEGER, INTENT(IN) :: KFROM, KTO
REAL(KIND=JPRB) :: ZHOOK_HANDLE
!
! Save current state for allocated arrays
IF (LKFROM) THEN
DIAG_MISC_ISBA_MODEL(KFROM)%XHV=>XHV
DIAG_MISC_ISBA_MODEL(KFROM)%XSWI=>XSWI
DIAG_MISC_ISBA_MODEL(KFROM)%XTSWI=>XTSWI
DIAG_MISC_ISBA_MODEL(KFROM)%XALT=>XALT
DIAG_MISC_ISBA_MODEL(KFROM)%XFLT=>XFLT
DIAG_MISC_ISBA_MODEL(KFROM)%XTWSNOW=>XTWSNOW
DIAG_MISC_ISBA_MODEL(KFROM)%XTDSNOW=>XTDSNOW
DIAG_MISC_ISBA_MODEL(KFROM)%XTTSNOW=>XTTSNOW
DIAG_MISC_ISBA_MODEL(KFROM)%XSNOWLIQ=>XSNOWLIQ
DIAG_MISC_ISBA_MODEL(KFROM)%XSNOWTEMP=>XSNOWTEMP
DIAG_MISC_ISBA_MODEL(KFROM)%XDPSNG=>XDPSNG
DIAG_MISC_ISBA_MODEL(KFROM)%XDPSNV=>XDPSNV
DIAG_MISC_ISBA_MODEL(KFROM)%XDPSN=>XDPSN
DIAG_MISC_ISBA_MODEL(KFROM)%XSEUIL=>XSEUIL
DIAG_MISC_ISBA_MODEL(KFROM)%XFAPAR=>XFAPAR
DIAG_MISC_ISBA_MODEL(KFROM)%XFAPIR=>XFAPIR
DIAG_MISC_ISBA_MODEL(KFROM)%XDFAPARC=>XDFAPARC
DIAG_MISC_ISBA_MODEL(KFROM)%XDFAPIRC=>XDFAPIRC
DIAG_MISC_ISBA_MODEL(KFROM)%XFAPAR_BS=>XFAPAR_BS
DIAG_MISC_ISBA_MODEL(KFROM)%XFAPIR_BS=>XFAPIR_BS
DIAG_MISC_ISBA_MODEL(KFROM)%XDLAI_EFFC=>XDLAI_EFFC
DIAG_MISC_ISBA_MODEL(KFROM)%XALBT=>XALBT
DIAG_MISC_ISBA_MODEL(KFROM)%XDFSAT=>XDFSAT
DIAG_MISC_ISBA_MODEL(KFROM)%XDFFG=>XDFFG
DIAG_MISC_ISBA_MODEL(KFROM)%XDFFV=>XDFFV
DIAG_MISC_ISBA_MODEL(KFROM)%XDFF=>XDFF
!
DIAG_MISC_ISBA_MODEL(KFROM)%XAVG_HV=>XAVG_HV
DIAG_MISC_ISBA_MODEL(KFROM)%XAVG_LAI=>XAVG_LAI
DIAG_MISC_ISBA_MODEL(KFROM)%XAVG_SWI=>XAVG_SWI
DIAG_MISC_ISBA_MODEL(KFROM)%XAVG_TSWI=>XAVG_TSWI
DIAG_MISC_ISBA_MODEL(KFROM)%XSOIL_TSWI=>XSOIL_TSWI
DIAG_MISC_ISBA_MODEL(KFROM)%XSOIL_TWG=>XSOIL_TWG
DIAG_MISC_ISBA_MODEL(KFROM)%XSOIL_TWGI=>XSOIL_TWGI
DIAG_MISC_ISBA_MODEL(KFROM)%XAVG_ALT=>XAVG_ALT
DIAG_MISC_ISBA_MODEL(KFROM)%XAVG_FLT=>XAVG_FLT
DIAG_MISC_ISBA_MODEL(KFROM)%XAVG_TWSNOW=>XAVG_TWSNOW
DIAG_MISC_ISBA_MODEL(KFROM)%XAVG_TDSNOW=>XAVG_TDSNOW
DIAG_MISC_ISBA_MODEL(KFROM)%XAVG_TTSNOW=>XAVG_TTSNOW
DIAG_MISC_ISBA_MODEL(KFROM)%XAVG_PSNG=>XAVG_PSNG
DIAG_MISC_ISBA_MODEL(KFROM)%XAVG_PSNV=>XAVG_PSNV
DIAG_MISC_ISBA_MODEL(KFROM)%XAVG_PSN=>XAVG_PSN
DIAG_MISC_ISBA_MODEL(KFROM)%XAVG_ALBT=>XAVG_ALBT
DIAG_MISC_ISBA_MODEL(KFROM)%XAVG_FSAT=>XAVG_FSAT
DIAG_MISC_ISBA_MODEL(KFROM)%XAVG_FFG=>XAVG_FFG
DIAG_MISC_ISBA_MODEL(KFROM)%XAVG_FFV=>XAVG_FFV
DIAG_MISC_ISBA_MODEL(KFROM)%XAVG_FF=>XAVG_FF
!
DIAG_MISC_ISBA_MODEL(KFROM)%XSURF_TSWI=>XSURF_TSWI
DIAG_MISC_ISBA_MODEL(KFROM)%XSURF_TWG=>XSURF_TWG
DIAG_MISC_ISBA_MODEL(KFROM)%XSURF_TWGI=>XSURF_TWGI
DIAG_MISC_ISBA_MODEL(KFROM)%XROOT_TSWI=>XROOT_TSWI
DIAG_MISC_ISBA_MODEL(KFROM)%XROOT_TWG=>XROOT_TWG
DIAG_MISC_ISBA_MODEL(KFROM)%XROOT_TWGI=>XROOT_TWGI
DIAG_MISC_ISBA_MODEL(KFROM)%XFRD2_TSWI=>XFRD2_TSWI
DIAG_MISC_ISBA_MODEL(KFROM)%XFRD2_TWG=>XFRD2_TWG
DIAG_MISC_ISBA_MODEL(KFROM)%XFRD2_TWGI=>XFRD2_TWGI
DIAG_MISC_ISBA_MODEL(KFROM)%XFRD3_TSWI=>XFRD3_TSWI
DIAG_MISC_ISBA_MODEL(KFROM)%XFRD3_TWG=>XFRD3_TWG
DIAG_MISC_ISBA_MODEL(KFROM)%XFRD3_TWGI=>XFRD3_TWGI
!
ENDIF
!
!
! Current model is set to model KTO
IF (LHOOK) CALL DR_HOOK('MODD_DIAG_MISC_ISBA_N:DIAG_MISC_ISBA_GOTO_MODEL',0,ZHOOK_HANDLE)
LSURF_MISC_BUDGET=>DIAG_MISC_ISBA_MODEL(KTO)%LSURF_MISC_BUDGET
LSURF_DIAG_ALBEDO=>DIAG_MISC_ISBA_MODEL(KTO)%LSURF_DIAG_ALBEDO
LSURF_MISC_DIF=>DIAG_MISC_ISBA_MODEL(KTO)%LSURF_MISC_DIF
!
XHV=>DIAG_MISC_ISBA_MODEL(KTO)%XHV
XSWI=>DIAG_MISC_ISBA_MODEL(KTO)%XSWI
XTSWI=>DIAG_MISC_ISBA_MODEL(KTO)%XTSWI
XALT=>DIAG_MISC_ISBA_MODEL(KTO)%XALT
XFLT=>DIAG_MISC_ISBA_MODEL(KTO)%XFLT
XTWSNOW=>DIAG_MISC_ISBA_MODEL(KTO)%XTWSNOW
XTDSNOW=>DIAG_MISC_ISBA_MODEL(KTO)%XTDSNOW
XTTSNOW=>DIAG_MISC_ISBA_MODEL(KTO)%XTTSNOW
XSNOWLIQ=>DIAG_MISC_ISBA_MODEL(KTO)%XSNOWLIQ
XSNOWTEMP=>DIAG_MISC_ISBA_MODEL(KTO)%XSNOWTEMP
XDPSNG=>DIAG_MISC_ISBA_MODEL(KTO)%XDPSNG
XDPSNV=>DIAG_MISC_ISBA_MODEL(KTO)%XDPSNV
XDPSN=>DIAG_MISC_ISBA_MODEL(KTO)%XDPSN
XSEUIL=>DIAG_MISC_ISBA_MODEL(KTO)%XSEUIL
XFAPAR=>DIAG_MISC_ISBA_MODEL(KTO)%XFAPAR
XFAPIR=>DIAG_MISC_ISBA_MODEL(KTO)%XFAPIR
XDFAPARC=>DIAG_MISC_ISBA_MODEL(KTO)%XDFAPARC
XDFAPIRC=>DIAG_MISC_ISBA_MODEL(KTO)%XDFAPIRC
XFAPAR_BS=>DIAG_MISC_ISBA_MODEL(KTO)%XFAPAR_BS
XFAPIR_BS=>DIAG_MISC_ISBA_MODEL(KTO)%XFAPIR_BS
XDLAI_EFFC=>DIAG_MISC_ISBA_MODEL(KTO)%XDLAI_EFFC
XALBT=>DIAG_MISC_ISBA_MODEL(KTO)%XALBT
XDFSAT=>DIAG_MISC_ISBA_MODEL(KTO)%XDFSAT
XDFFG=>DIAG_MISC_ISBA_MODEL(KTO)%XDFFG
XDFFV=>DIAG_MISC_ISBA_MODEL(KTO)%XDFFV
XDFF=>DIAG_MISC_ISBA_MODEL(KTO)%XDFF
!
XAVG_HV=>DIAG_MISC_ISBA_MODEL(KTO)%XAVG_HV
XAVG_LAI=>DIAG_MISC_ISBA_MODEL(KTO)%XAVG_LAI
XAVG_SWI=>DIAG_MISC_ISBA_MODEL(KTO)%XAVG_SWI
XAVG_TSWI=>DIAG_MISC_ISBA_MODEL(KTO)%XAVG_TSWI
XSOIL_TSWI=>DIAG_MISC_ISBA_MODEL(KTO)%XSOIL_TSWI
XSOIL_TWG=>DIAG_MISC_ISBA_MODEL(KTO)%XSOIL_TWG
XSOIL_TWGI=>DIAG_MISC_ISBA_MODEL(KTO)%XSOIL_TWGI
XAVG_ALT=>DIAG_MISC_ISBA_MODEL(KTO)%XAVG_ALT
XAVG_FLT=>DIAG_MISC_ISBA_MODEL(KTO)%XAVG_FLT
XAVG_TWSNOW=>DIAG_MISC_ISBA_MODEL(KTO)%XAVG_TWSNOW
XAVG_TDSNOW=>DIAG_MISC_ISBA_MODEL(KTO)%XAVG_TDSNOW
XAVG_TTSNOW=>DIAG_MISC_ISBA_MODEL(KTO)%XAVG_TTSNOW
XAVG_PSNG=>DIAG_MISC_ISBA_MODEL(KTO)%XAVG_PSNG
XAVG_PSNV=>DIAG_MISC_ISBA_MODEL(KTO)%XAVG_PSNV
XAVG_PSN=>DIAG_MISC_ISBA_MODEL(KTO)%XAVG_PSN
XAVG_ALBT=>DIAG_MISC_ISBA_MODEL(KTO)%XAVG_ALBT
XAVG_FSAT=>DIAG_MISC_ISBA_MODEL(KTO)%XAVG_FSAT
XAVG_FFG=>DIAG_MISC_ISBA_MODEL(KTO)%XAVG_FFG
XAVG_FFV=>DIAG_MISC_ISBA_MODEL(KTO)%XAVG_FFV
XAVG_FF=>DIAG_MISC_ISBA_MODEL(KTO)%XAVG_FF
!
XSURF_TSWI=>DIAG_MISC_ISBA_MODEL(KTO)%XSURF_TSWI
XSURF_TWG=>DIAG_MISC_ISBA_MODEL(KTO)%XSURF_TWG
XSURF_TWGI=>DIAG_MISC_ISBA_MODEL(KTO)%XSURF_TWGI
XROOT_TSWI=>DIAG_MISC_ISBA_MODEL(KTO)%XROOT_TSWI
XROOT_TWG=>DIAG_MISC_ISBA_MODEL(KTO)%XROOT_TWG
XROOT_TWGI=>DIAG_MISC_ISBA_MODEL(KTO)%XROOT_TWGI
XFRD2_TSWI=>DIAG_MISC_ISBA_MODEL(KTO)%XFRD2_TSWI
XFRD2_TWG=>DIAG_MISC_ISBA_MODEL(KTO)%XFRD2_TWG
XFRD2_TWGI=>DIAG_MISC_ISBA_MODEL(KTO)%XFRD2_TWGI
XFRD3_TSWI=>DIAG_MISC_ISBA_MODEL(KTO)%XFRD3_TSWI
XFRD3_TWG=>DIAG_MISC_ISBA_MODEL(KTO)%XFRD3_TWG
XFRD3_TWGI=>DIAG_MISC_ISBA_MODEL(KTO)%XFRD3_TWGI
!
IF (LHOOK) CALL DR_HOOK('MODD_DIAG_MISC_ISBA_N:DIAG_MISC_ISBA_GOTO_MODEL',1,ZHOOK_HANDLE)
!

END SUBROUTINE DIAG_MISC_ISBA_GOTO_MODEL

SUBROUTINE DIAG_MISC_ISBA_ALLOC(KMODEL)
INTEGER, INTENT(IN) :: KMODEL
INTEGER :: J
REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK("MODD_DIAG_MISC_ISBA_N:DIAG_MISC_ISBA_ALLOC",0,ZHOOK_HANDLE)
ALLOCATE(DIAG_MISC_ISBA_MODEL(KMODEL))
DO J=1,KMODEL
  NULLIFY(DIAG_MISC_ISBA_MODEL(J)%XHV)
  NULLIFY(DIAG_MISC_ISBA_MODEL(J)%XSWI)
  NULLIFY(DIAG_MISC_ISBA_MODEL(J)%XTSWI)
  NULLIFY(DIAG_MISC_ISBA_MODEL(J)%XALT)
  NULLIFY(DIAG_MISC_ISBA_MODEL(J)%XFLT)
  NULLIFY(DIAG_MISC_ISBA_MODEL(J)%XSNOWLIQ)
  NULLIFY(DIAG_MISC_ISBA_MODEL(J)%XSNOWTEMP)
  NULLIFY(DIAG_MISC_ISBA_MODEL(J)%XTWSNOW)
  NULLIFY(DIAG_MISC_ISBA_MODEL(J)%XTDSNOW)
  NULLIFY(DIAG_MISC_ISBA_MODEL(J)%XTTSNOW)
  NULLIFY(DIAG_MISC_ISBA_MODEL(J)%XDPSNG)
  NULLIFY(DIAG_MISC_ISBA_MODEL(J)%XDPSNV)
  NULLIFY(DIAG_MISC_ISBA_MODEL(J)%XDPSN)
  NULLIFY(DIAG_MISC_ISBA_MODEL(J)%XALBT)
  NULLIFY(DIAG_MISC_ISBA_MODEL(J)%XDFFG)
  NULLIFY(DIAG_MISC_ISBA_MODEL(J)%XDFFV)
  NULLIFY(DIAG_MISC_ISBA_MODEL(J)%XDFF)
  NULLIFY(DIAG_MISC_ISBA_MODEL(J)%XSEUIL)
  NULLIFY(DIAG_MISC_ISBA_MODEL(J)%XFAPAR)
  NULLIFY(DIAG_MISC_ISBA_MODEL(J)%XFAPIR)
  NULLIFY(DIAG_MISC_ISBA_MODEL(J)%XDFAPARC)
  NULLIFY(DIAG_MISC_ISBA_MODEL(J)%XDFAPIRC)
  NULLIFY(DIAG_MISC_ISBA_MODEL(J)%XFAPAR_BS)
  NULLIFY(DIAG_MISC_ISBA_MODEL(J)%XFAPIR_BS)
  NULLIFY(DIAG_MISC_ISBA_MODEL(J)%XDLAI_EFFC)  
  NULLIFY(DIAG_MISC_ISBA_MODEL(J)%XAVG_HV)
  NULLIFY(DIAG_MISC_ISBA_MODEL(J)%XAVG_LAI)
  NULLIFY(DIAG_MISC_ISBA_MODEL(J)%XAVG_SWI)
  NULLIFY(DIAG_MISC_ISBA_MODEL(J)%XAVG_TSWI)
  NULLIFY(DIAG_MISC_ISBA_MODEL(J)%XSOIL_TSWI)
  NULLIFY(DIAG_MISC_ISBA_MODEL(J)%XSOIL_TWG)
  NULLIFY(DIAG_MISC_ISBA_MODEL(J)%XSOIL_TWGI)
  NULLIFY(DIAG_MISC_ISBA_MODEL(J)%XAVG_ALT)
  NULLIFY(DIAG_MISC_ISBA_MODEL(J)%XAVG_FLT)
  NULLIFY(DIAG_MISC_ISBA_MODEL(J)%XAVG_TWSNOW)
  NULLIFY(DIAG_MISC_ISBA_MODEL(J)%XAVG_TDSNOW)
  NULLIFY(DIAG_MISC_ISBA_MODEL(J)%XAVG_TTSNOW)
  NULLIFY(DIAG_MISC_ISBA_MODEL(J)%XAVG_PSNG)
  NULLIFY(DIAG_MISC_ISBA_MODEL(J)%XAVG_PSNV)
  NULLIFY(DIAG_MISC_ISBA_MODEL(J)%XAVG_PSN)
  NULLIFY(DIAG_MISC_ISBA_MODEL(J)%XAVG_ALBT)
  NULLIFY(DIAG_MISC_ISBA_MODEL(J)%XAVG_FFG)
  NULLIFY(DIAG_MISC_ISBA_MODEL(J)%XAVG_FFV)
  NULLIFY(DIAG_MISC_ISBA_MODEL(J)%XAVG_FF)
  NULLIFY(DIAG_MISC_ISBA_MODEL(J)%XSURF_TSWI)
  NULLIFY(DIAG_MISC_ISBA_MODEL(J)%XSURF_TWG)
  NULLIFY(DIAG_MISC_ISBA_MODEL(J)%XSURF_TWGI)
  NULLIFY(DIAG_MISC_ISBA_MODEL(J)%XROOT_TSWI)
  NULLIFY(DIAG_MISC_ISBA_MODEL(J)%XROOT_TWG)
  NULLIFY(DIAG_MISC_ISBA_MODEL(J)%XROOT_TWGI)
  NULLIFY(DIAG_MISC_ISBA_MODEL(J)%XFRD2_TSWI)
  NULLIFY(DIAG_MISC_ISBA_MODEL(J)%XFRD2_TWG)
  NULLIFY(DIAG_MISC_ISBA_MODEL(J)%XFRD2_TWGI)
  NULLIFY(DIAG_MISC_ISBA_MODEL(J)%XFRD3_TSWI)
  NULLIFY(DIAG_MISC_ISBA_MODEL(J)%XFRD3_TWG)
  NULLIFY(DIAG_MISC_ISBA_MODEL(J)%XFRD3_TWGI)  
ENDDO
DIAG_MISC_ISBA_MODEL(:)%LSURF_MISC_BUDGET=.FALSE.
DIAG_MISC_ISBA_MODEL(:)%LSURF_DIAG_ALBEDO=.FALSE.
DIAG_MISC_ISBA_MODEL(:)%LSURF_MISC_DIF=.FALSE.
IF (LHOOK) CALL DR_HOOK("MODD_DIAG_MISC_ISBA_N:DIAG_MISC_ISBA_ALLOC",1,ZHOOK_HANDLE)
END SUBROUTINE DIAG_MISC_ISBA_ALLOC

SUBROUTINE DIAG_MISC_ISBA_DEALLO
REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK("MODD_DIAG_MISC_ISBA_N:DIAG_MISC_ISBA_DEALLO",0,ZHOOK_HANDLE)
IF (ALLOCATED(DIAG_MISC_ISBA_MODEL)) DEALLOCATE(DIAG_MISC_ISBA_MODEL)
IF (LHOOK) CALL DR_HOOK("MODD_DIAG_MISC_ISBA_N:DIAG_MISC_ISBA_DEALLO",1,ZHOOK_HANDLE)
END SUBROUTINE DIAG_MISC_ISBA_DEALLO

END MODULE MODD_DIAG_MISC_ISBA_n
