!SURFEX_LIC Copyright 1994-2014 Meteo-France 
!SURFEX_LIC This is part of the SURFEX software governed by the CeCILL-C  licence
!SURFEX_LIC version 1. See LICENSE, CeCILL-C_V1-en.txt and CeCILL-C_V1-fr.txt
!SURFEX_LIC for details. version 1.
!     ################
      MODULE MODD_SSO_CANOPY_n
!     ################
!
!!****  *MODD_SSO_CANOPY_n - declaration of surface parameters for
!                                orographic canopy
!!
!!    PURPOSE
!!    -------
!     Declaration of surface parameters
!
!!
!!**  IMPLICIT ARGUMENTS
!!    ------------------
!!      None 
!!
!!    REFERENCE
!!    ---------
!!
!!    AUTHOR
!!    ------
!!	V. Masson   *Meteo France*
!!
!!    MODIFICATIONS
!!    -------------
!!      Original       07/2006
!
!*       0.   DECLARATIONS
!             ------------
!
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK
USE PARKIND1  ,ONLY : JPRB
!
IMPLICIT NONE

TYPE SSO_CANOPY_t
!
INTEGER                       :: NLVL ! number      of levels in canopy
REAL, POINTER, DIMENSION(:,:) :: XZ   ! height of middle of each level grid   (m)
REAL, POINTER, DIMENSION(:,:) :: XU   ! wind        at each level in canopy   (m/s)
REAL, POINTER, DIMENSION(:,:) :: XTKE ! Tke         at each level in canopy   (m2/s2)
!
REAL, POINTER, DIMENSION(:,:) :: XDZ  ! depth       of each level in canopy   (m)
REAL, POINTER, DIMENSION(:,:) :: XZF  ! height of bottom of each level grid   (m)

REAL, POINTER, DIMENSION(:,:) :: XDZF ! depth between  each level in canopy   (m)
!
END TYPE SSO_CANOPY_t

TYPE(SSO_CANOPY_t), ALLOCATABLE, TARGET, SAVE :: SSO_CANOPY_MODEL(:)

INTEGER, POINTER :: NLVL=>NULL()
!$OMP THREADPRIVATE(NLVL)
REAL, POINTER, DIMENSION(:,:) :: XZ=>NULL()
!$OMP THREADPRIVATE(XZ)
REAL, POINTER, DIMENSION(:,:) :: XU=>NULL()
!$OMP THREADPRIVATE(XU)
REAL, POINTER, DIMENSION(:,:) :: XTKE=>NULL()
!$OMP THREADPRIVATE(XTKE)
REAL, POINTER, DIMENSION(:,:) :: XDZ=>NULL()
!$OMP THREADPRIVATE(XDZ)
REAL, POINTER, DIMENSION(:,:) :: XZF=>NULL()
!$OMP THREADPRIVATE(XZF)
REAL, POINTER, DIMENSION(:,:) :: XDZF=>NULL()
!$OMP THREADPRIVATE(XDZF)

CONTAINS

SUBROUTINE SSO_CANOPY_GOTO_MODEL(KFROM, KTO, LKFROM)
LOGICAL, INTENT(IN) :: LKFROM
INTEGER, INTENT(IN) :: KFROM, KTO
REAL(KIND=JPRB) :: ZHOOK_HANDLE
!
IF (LKFROM) THEN
! Save current state for allocated arrays
SSO_CANOPY_MODEL(KFROM)%XZ=>XZ
SSO_CANOPY_MODEL(KFROM)%XU=>XU
SSO_CANOPY_MODEL(KFROM)%XTKE=>XTKE
SSO_CANOPY_MODEL(KFROM)%XDZ=>XDZ
SSO_CANOPY_MODEL(KFROM)%XZF=>XZF
SSO_CANOPY_MODEL(KFROM)%XDZF=>XDZF
ENDIF
!
! Current model is set to model KTO
IF (LHOOK) CALL DR_HOOK('MODD_SSO_CANOPY_N:SSO_CANOPY_GOTO_MODEL',0,ZHOOK_HANDLE)
NLVL=>SSO_CANOPY_MODEL(KTO)%NLVL
XZ=>SSO_CANOPY_MODEL(KTO)%XZ
XU=>SSO_CANOPY_MODEL(KTO)%XU
XTKE=>SSO_CANOPY_MODEL(KTO)%XTKE
XDZ=>SSO_CANOPY_MODEL(KTO)%XDZ
XZF=>SSO_CANOPY_MODEL(KTO)%XZF
XDZF=>SSO_CANOPY_MODEL(KTO)%XDZF
IF (LHOOK) CALL DR_HOOK('MODD_SSO_CANOPY_N:SSO_CANOPY_GOTO_MODEL',1,ZHOOK_HANDLE)

END SUBROUTINE SSO_CANOPY_GOTO_MODEL


SUBROUTINE SSO_CANOPY_ALLOC(KMODEL)
INTEGER, INTENT(IN) :: KMODEL
INTEGER :: J
REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK("MODD_SSO_CANOPY_N:SSO_CANOPY_ALLOC",0,ZHOOK_HANDLE)
ALLOCATE(SSO_CANOPY_MODEL(KMODEL))
DO J=1,KMODEL
  NULLIFY(SSO_CANOPY_MODEL(J)%XZ)
  NULLIFY(SSO_CANOPY_MODEL(J)%XU)
  NULLIFY(SSO_CANOPY_MODEL(J)%XTKE)
  NULLIFY(SSO_CANOPY_MODEL(J)%XDZ)
  NULLIFY(SSO_CANOPY_MODEL(J)%XZF)
  NULLIFY(SSO_CANOPY_MODEL(J)%XDZF)
ENDDO
SSO_CANOPY_MODEL(:)%NLVL=0
IF (LHOOK) CALL DR_HOOK("MODD_SSO_CANOPY_N:SSO_CANOPY_ALLOC",1,ZHOOK_HANDLE)
END SUBROUTINE SSO_CANOPY_ALLOC

SUBROUTINE SSO_CANOPY_DEALLO
REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK("MODD_SSO_CANOPY_N:SSO_CANOPY_DEALLO",0,ZHOOK_HANDLE)
IF (ALLOCATED(SSO_CANOPY_MODEL)) DEALLOCATE(SSO_CANOPY_MODEL)
IF (LHOOK) CALL DR_HOOK("MODD_SSO_CANOPY_N:SSO_CANOPY_DEALLO",1,ZHOOK_HANDLE)
END SUBROUTINE SSO_CANOPY_DEALLO


END MODULE MODD_SSO_CANOPY_n
