!SURFEX_LIC Copyright 1994-2014 Meteo-France 
!SURFEX_LIC This is part of the SURFEX software governed by the CeCILL-C  licence
!SURFEX_LIC version 1. See LICENSE, CeCILL-C_V1-en.txt and CeCILL-C_V1-fr.txt
!SURFEX_LIC for details. version 1.
!     #########################
      SUBROUTINE READ_BLD_DESCRIPTION(HPROGRAM)
!     #########################
!
!!
!!    PURPOSE
!!    -------
!!
!!    METHOD
!!    ------
!!
!!
!!    EXTERNAL
!!    --------
!!
!!    IMPLICIT ARGUMENTS
!!    ------------------
!!
!!    REFERENCE
!!    ---------
!!
!!    AUTHOR
!!    ------
!!
!!    V. Masson        Meteo-France
!!
!!    MODIFICATION
!!    ------------
!!
!!    Original    05/2012 
!
!----------------------------------------------------------------------------
!
!*    0.     DECLARATION
!            -----------
!
USE MODD_BLD_DESCRIPTION
!
USE MODI_READ_SURF
USE MODI_ABOR1_SFX
!
!
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK
USE PARKIND1  ,ONLY : JPRB
!
IMPLICIT NONE
!
!*    0.1    Declaration of arguments
!            ------------------------
!
 CHARACTER(LEN=6),  INTENT(IN) :: HPROGRAM
!
!
!*    0.2    Declaration of local variables
!      ------------------------------
!
REAL(KIND=JPRB) :: ZHOOK_HANDLE
!
REAL, DIMENSION(:), ALLOCATABLE :: ZWORK
INTEGER                         :: IRESP
INTEGER                         :: I1, I2
INTEGER                         :: JL
INTEGER                         :: ITOT
!-------------------------------------------------------------------------------
!-------------------------------------------------------------------------------
!
IF (LHOOK) CALL DR_HOOK('READ_BLD_DESCRIPTION',0,ZHOOK_HANDLE)
!
!-------------------------------------------------------------------------------
!
!*    1.   Read configuration variables of the descriptive data
!          ----------------------------------------------------
!
ALLOCATE(ZWORK(7))
 CALL READ_SURF(HPROGRAM,'BLD_DESC_CNF',ZWORK,IRESP,HDIR='-')
!
NDESC_BLD         = NINT(ZWORK(1))
NDESC_AGE         = NINT(ZWORK(2))
NDESC_USE         = NINT(ZWORK(3))
NDESC_WALL_LAYER  = NINT(ZWORK(4))
NDESC_ROOF_LAYER  = NINT(ZWORK(5))
NDESC_ROAD_LAYER  = NINT(ZWORK(6))
NDESC_FLOOR_LAYER = NINT(ZWORK(7))
!
DEALLOCATE(ZWORK)
!
NDESC_CODE = NDESC_BLD * NDESC_AGE
!-------------------------------------------------------------------------------
!
!*    2.   Allocates descriptive data
!          --------------------------
!
!
ALLOCATE(NDESC_BLD_LIST(NDESC_BLD))
ALLOCATE(NDESC_CODE_LIST(NDESC_CODE))
ALLOCATE(XDESC_ALB_ROOF(NDESC_CODE))
ALLOCATE(XDESC_ALB_ROAD(NDESC_CODE))
ALLOCATE(XDESC_ALB_WALL(NDESC_CODE))
ALLOCATE(XDESC_EMIS_ROOF(NDESC_CODE))
ALLOCATE(XDESC_EMIS_ROAD(NDESC_CODE))
ALLOCATE(XDESC_EMIS_WALL(NDESC_CODE))
ALLOCATE(XDESC_HC_ROOF(NDESC_CODE,NDESC_ROOF_LAYER))
ALLOCATE(XDESC_TC_ROOF(NDESC_CODE,NDESC_ROOF_LAYER))
ALLOCATE(XDESC_D_ROOF (NDESC_CODE,NDESC_ROOF_LAYER))
ALLOCATE(XDESC_HC_ROAD(NDESC_CODE,NDESC_ROAD_LAYER))
ALLOCATE(XDESC_TC_ROAD(NDESC_CODE,NDESC_ROAD_LAYER))
ALLOCATE(XDESC_D_ROAD (NDESC_CODE,NDESC_ROAD_LAYER))
ALLOCATE(XDESC_HC_WALL(NDESC_CODE,NDESC_WALL_LAYER))
ALLOCATE(XDESC_TC_WALL(NDESC_CODE,NDESC_WALL_LAYER))
ALLOCATE(XDESC_D_WALL (NDESC_CODE,NDESC_WALL_LAYER))
ALLOCATE(XDESC_HC_FLOOR(NDESC_CODE,NDESC_FLOOR_LAYER))
ALLOCATE(XDESC_TC_FLOOR(NDESC_CODE,NDESC_FLOOR_LAYER))
ALLOCATE(XDESC_D_FLOOR (NDESC_CODE,NDESC_FLOOR_LAYER))
ALLOCATE(XDESC_SHGC(NDESC_CODE))
ALLOCATE(XDESC_U_WIN(NDESC_CODE))
ALLOCATE(XDESC_GR(NDESC_CODE))
!
ALLOCATE(XDESC_F_WASTE_CAN(NDESC_CODE))
ALLOCATE(XDESC_F_WATER_COND(NDESC_CODE))
ALLOCATE(XDESC_COP_RAT(NDESC_CODE))
ALLOCATE(XDESC_EFF_HEAT(NDESC_CODE))
ALLOCATE(XDESC_INF(NDESC_CODE))
ALLOCATE(XDESC_V_VENT(NDESC_CODE))
ALLOCATE(XDESC_GREENROOF(NDESC_CODE))
!
!
ALLOCATE(NDESC_USE_LIST(NDESC_USE))
ALLOCATE(XDESC_TCOOL_TARGET(NDESC_USE))
ALLOCATE(XDESC_THEAT_TARGET(NDESC_USE))
ALLOCATE(XDESC_QIN(NDESC_USE))
ALLOCATE(XDESC_QIN_FLAT(NDESC_USE))
ALLOCATE(XDESC_SHGC_SH(NDESC_USE))
ALLOCATE(XDESC_SHADE(NDESC_USE))
ALLOCATE(XDESC_NATVENT(NDESC_USE))
!
ALLOCATE(NDESC_AGE_LIST(NDESC_AGE))
ALLOCATE(NDESC_AGE_DATE(NDESC_AGE))
!-------------------------------------------------------------------------------
!
!*    3.   Read descriptive data
!          ---------------------
!
ITOT=(17+3*NDESC_ROOF_LAYER+3*NDESC_ROAD_LAYER+3*NDESC_WALL_LAYER+3*NDESC_FLOOR_LAYER)*NDESC_CODE &
      + 8*NDESC_USE + 2*NDESC_AGE + NDESC_BLD
ALLOCATE(ZWORK(ITOT))
!
 CALL READ_SURF(HPROGRAM,'BLD_DESC_DAT',ZWORK,IRESP,HDIR='-')
!
!
I1=0 ; I2=0
 CALL UP_DESC_IND(NDESC_BLD)  ; NDESC_BLD_LIST(:)  = NINT(ZWORK(I1:I2))
 CALL UP_DESC_IND(NDESC_CODE) ; NDESC_CODE_LIST(:)  = NINT(ZWORK(I1:I2))
 CALL UP_DESC_IND(NDESC_CODE) ; XDESC_ALB_ROOF(:)  = ZWORK(I1:I2)
 CALL UP_DESC_IND(NDESC_CODE) ; XDESC_ALB_ROAD(:)  = ZWORK(I1:I2)
 CALL UP_DESC_IND(NDESC_CODE) ; XDESC_ALB_WALL(:)  = ZWORK(I1:I2)
 CALL UP_DESC_IND(NDESC_CODE) ; XDESC_EMIS_ROOF(:) = ZWORK(I1:I2)
 CALL UP_DESC_IND(NDESC_CODE) ; XDESC_EMIS_ROAD(:) = ZWORK(I1:I2)
 CALL UP_DESC_IND(NDESC_CODE) ; XDESC_EMIS_WALL(:) = ZWORK(I1:I2)
DO JL=1,NDESC_ROOF_LAYER
  CALL UP_DESC_IND(NDESC_CODE) ; XDESC_HC_ROOF(:,JL) = ZWORK(I1:I2)
END DO
DO JL=1,NDESC_ROOF_LAYER
  CALL UP_DESC_IND(NDESC_CODE) ; XDESC_TC_ROOF(:,JL) = ZWORK(I1:I2)
END DO
DO JL=1,NDESC_ROOF_LAYER
  CALL UP_DESC_IND(NDESC_CODE) ; XDESC_D_ROOF (:,JL) = ZWORK(I1:I2)
END DO
DO JL=1,NDESC_ROAD_LAYER
  CALL UP_DESC_IND(NDESC_CODE) ; XDESC_HC_ROAD(:,JL) = ZWORK(I1:I2)
END DO
DO JL=1,NDESC_ROAD_LAYER
  CALL UP_DESC_IND(NDESC_CODE) ; XDESC_TC_ROAD(:,JL) = ZWORK(I1:I2)
END DO
DO JL=1,NDESC_ROAD_LAYER
  CALL UP_DESC_IND(NDESC_CODE) ; XDESC_D_ROAD (:,JL) = ZWORK(I1:I2)
END DO
DO JL=1,NDESC_WALL_LAYER
  CALL UP_DESC_IND(NDESC_CODE) ; XDESC_HC_WALL(:,JL) = ZWORK(I1:I2)
END DO
DO JL=1,NDESC_WALL_LAYER
  CALL UP_DESC_IND(NDESC_CODE) ; XDESC_TC_WALL(:,JL) = ZWORK(I1:I2)
END DO
DO JL=1,NDESC_WALL_LAYER
  CALL UP_DESC_IND(NDESC_CODE) ; XDESC_D_WALL (:,JL) = ZWORK(I1:I2)
END DO
DO JL=1,NDESC_FLOOR_LAYER
  CALL UP_DESC_IND(NDESC_CODE) ; XDESC_HC_FLOOR(:,JL) = ZWORK(I1:I2)
END DO
DO JL=1,NDESC_FLOOR_LAYER
  CALL UP_DESC_IND(NDESC_CODE) ; XDESC_TC_FLOOR(:,JL) = ZWORK(I1:I2)
END DO
DO JL=1,NDESC_FLOOR_LAYER
  CALL UP_DESC_IND(NDESC_CODE) ; XDESC_D_FLOOR (:,JL) = ZWORK(I1:I2)
END DO
!
 CALL UP_DESC_IND(NDESC_CODE) ; XDESC_SHGC(:)  = ZWORK(I1:I2)
 CALL UP_DESC_IND(NDESC_CODE) ; XDESC_U_WIN(:) = ZWORK(I1:I2)
 CALL UP_DESC_IND(NDESC_CODE) ; XDESC_GR(:)    = ZWORK(I1:I2)
!
 CALL UP_DESC_IND(NDESC_CODE) ; XDESC_F_WASTE_CAN(:)  = ZWORK(I1:I2)
 CALL UP_DESC_IND(NDESC_CODE) ; XDESC_F_WATER_COND(:) = ZWORK(I1:I2)
 CALL UP_DESC_IND(NDESC_CODE) ; XDESC_COP_RAT(:)      = ZWORK(I1:I2)
 CALL UP_DESC_IND(NDESC_CODE) ; XDESC_EFF_HEAT(:)     = ZWORK(I1:I2)
 CALL UP_DESC_IND(NDESC_CODE) ; XDESC_INF(:)          = ZWORK(I1:I2)
 CALL UP_DESC_IND(NDESC_CODE) ; XDESC_V_VENT(:)       = ZWORK(I1:I2)
 CALL UP_DESC_IND(NDESC_CODE) ; XDESC_GREENROOF(:)    = ZWORK(I1:I2)
!
 CALL UP_DESC_IND(NDESC_USE) ; NDESC_USE_LIST(:)     = NINT(ZWORK(I1:I2))
 CALL UP_DESC_IND(NDESC_USE) ; XDESC_TCOOL_TARGET(:) = ZWORK(I1:I2)
 CALL UP_DESC_IND(NDESC_USE) ; XDESC_THEAT_TARGET(:) = ZWORK(I1:I2)
 CALL UP_DESC_IND(NDESC_USE) ; XDESC_QIN(:)          = ZWORK(I1:I2)
 CALL UP_DESC_IND(NDESC_USE) ; XDESC_QIN_FLAT(:)     = ZWORK(I1:I2)
 CALL UP_DESC_IND(NDESC_USE) ; XDESC_SHGC_SH(:)      = ZWORK(I1:I2)
 CALL UP_DESC_IND(NDESC_USE) ; XDESC_SHADE(:)        = ZWORK(I1:I2)
 CALL UP_DESC_IND(NDESC_USE) ; XDESC_NATVENT(:)      = ZWORK(I1:I2)
!
 CALL UP_DESC_IND(NDESC_AGE) ; NDESC_AGE_LIST(:)     = NINT(ZWORK(I1:I2))
 CALL UP_DESC_IND(NDESC_AGE) ; NDESC_AGE_DATE(:)     = NINT(ZWORK(I1:I2))
!
DEALLOCATE(ZWORK)
!
IF (LHOOK) CALL DR_HOOK('READ_BLD_DESCRIPTION',1,ZHOOK_HANDLE)
!-------------------------------------------------------------------------------
CONTAINS
SUBROUTINE UP_DESC_IND(K)
INTEGER, INTENT(IN) :: K
I1=I2+1
I2=I2+K
END SUBROUTINE UP_DESC_IND
!-------------------------------------------------------------------------------
!
END SUBROUTINE READ_BLD_DESCRIPTION
